#include <ArduinoBLE.h>
#include <Arduino_BHY2.h>
#include "Nicla_System.h"
#include <Arduino.h>

SensorXYZ accel(SENSOR_ID_LACC);
SensorXYZ gyro(SENSOR_ID_GYRO);

short accelXarray[18];
short accelYarray[18];
short accelZarray[18];
short gyroXarray[18];
short gyroYarray[18];
short gyroZarray[18];



typedef struct settings_t {
  byte hyzer;
  byte direct;
  byte height;
  short maxspd;
  short maxrev;
};
const int union_size = sizeof(settings_t);
typedef union btPacket_t {
  settings_t structure;
  byte byteArray[union_size];
};

btPacket_t settings;


// address 40:3A:1F:D6:6F:E1

BLEService throwData("19B10000-E8F2-537E-4F6C-D104768A1214");
//BLEShortCharacteristic throwChar("19B10001-E8F2-537E-4F6C-D104768A1214", BLERead | BLENotify);
BLECharacteristic throwStat("19B10006-E8F2-537E-4F6C-D104768A1214", BLERead | BLENotify, sizeof settings.byteArray);
//BLEStringCharacteristic trig("19B10004-E8F2-537E-4F6C-D104768A1214", BLEWrite, 1);


void setup()

{
  Serial.begin(9600);
  while (!Serial);
  BHY2.begin();
  accel.begin();
  gyro.begin();
  nicla::begin();
  nicla::leds.begin();
  SensorConfig cfg = accel.getConfiguration();
  accel.setRange(16);                  //this sets the range of accel to +/-16g,
  if (!BLE.begin()) {
    Serial.println("BLE failed");
    while (1);
  }
  BLE.setLocalName("DGT");
  BLE.setAdvertisedService(throwData);
  //throwData.addCharacteristic(throwChar);
  throwData.addCharacteristic(throwStat);
  //throwData.addCharacteristic(trig);
  throwStat.writeValue(settings.byteArray, sizeof settings.byteArray);
  BLE.addService(throwData);
  BLE.advertise();
  Serial.println("Setup complete");
  nicla::leds.setColor(red);            // RED LED indicates NO BLUETOOTH CONNECTION
}

void loop() {
  static auto printTime = millis();
  BLEDevice central = BLE.central();
  if (central) {
    //millis();
    Serial.println("Central Connected");      // GREEN LED indicates active Bluetooth connection
    while (central.connected()) {
      static auto printTime = millis();
      BHY2.update();
      nicla::leds.setColor(green);      // green LED indicates waiting on user
      if (abs(accel.y()) > 6000 && millis() - printTime >= 1000){          //Detect trigger for throw, once a significant amount of acceleration is observed start recording the throw
        printTime = millis();
        Serial.println("Trigger received");
        nicla::leds.setColor(blue);      // blue LED indicates in progress
        recordThrow();
        dataAccel();
        dataGyro();
        throwStat.writeValue(settings.byteArray, sizeof settings.byteArray);
        

      }
    }
  }

}

void recordThrow() {                    // Function to record the users throw into a series of arrays for each axis of gyroscope and accelerometer
  for (int i = 0; i <= 17; i++) {
    BHY2.update();
    accelXarray[i] = accel.x();
    accelYarray[i] = accel.y();
    accelZarray[i] = accel.z();
    gyroXarray[i] = gyro.x();
    gyroYarray[i] = gyro.y();
    gyroZarray[i] = gyro.z();
  }
  Serial.println("record complete");
}
void dataAccel() {
  short maxxVal = accelXarray[0];       // Declaring needed variables for analysis
  short maxyVal = accelYarray[0];
  short maxzVal = accelZarray[0];
  short minyVal = accelYarray[0];

  for (int i = 0; i < 17; i++) {                 //  Loop to process array for minimum and maximum values of each axis.
    if (accelXarray[i] > maxxVal) {
      maxxVal = accelXarray[i];
    }
    if (abs(accelYarray[i]) > maxyVal) {
      maxyVal = accelYarray[i];
    }
    if (accelZarray[i] > maxzVal) {
      maxzVal = accelZarray[i];
    }
 //    if (accelYarray[i] < minyVal) {
 //     minyVal = accelYarray[i];
 //   }
  }
  if (maxxVal < 1000) {                           // Criteria for throw too high     //Original Value 1,000
      settings.structure.height = 1;              // Update UI, throw too high
    }
  else if (maxxVal > 3500 ) {                       // Criteria for throw too Low     // Original Value 5,000
    settings.structure.height = 2;                     // Update UI, throw too low
  }
  else if (maxxVal < 3500 && maxxVal > 1000) {        // Criteria for good vertical release angle        // Original 5k & 1k
    settings.structure.height = 3;                  // Update UI, ok throw.
  }

  if (maxzVal > 3500) {                            // Criteria for rounding observed               // Original 1k 
    settings.structure.direct = 1;    
  }
  else if (maxzVal < 3500) {                         // Criteria for NO rounding observed        // Original = 5k
    settings.structure.direct = 2;
  }

  settings.structure.maxspd = round(abs(maxyVal) / 208.98);         // Update UI with max disc linear accel., convert to M/s

 // Serial.println((String)"A-Miny = " + minyVal);
  Serial.println((String)"A-Maxy = " + maxyVal);
  Serial.println((String)"A-Maxx = " + maxxVal);
  Serial.println((String)"A-Maxz = " + maxzVal);
  // Y-axis values for max accel
  // X-axis for vertical/ lift angle.
  // X-axis negative/low for too high
  // X-axis Large positive for too low, larger than Z?
  // X-axis low positive for just right. (approx equal to Z? )
  // Z-axis negative values identifies rounding
  // Z-axis positive large positive values not rounding

}

void dataGyro() {                           // Function to analyze data from gyroscope
  short maxxVal = gyroXarray[0];
  short maxyVal = gyroYarray[0];
  short maxzVal = gyroZarray[0];
  short minzVal = gyroZarray[0];
  short minyVal = gyroYarray[0];
  
  for (int i = 0; i < 17; i++) {             // Process array values for maximum of each axis.
    if (gyroXarray[i] > maxxVal) {
      maxxVal = gyroXarray[i];
    }
    if (gyroYarray[i] > maxyVal) {
      maxyVal = gyroYarray[i];
    }
    if (gyroZarray[i] > maxzVal) {
      maxzVal = gyroZarray[i];
    }
 //   if (gyroZarray[i] < minzVal) {
 //     minzVal = gyroZarray[i];
 //   }
  }
  settings.structure.maxrev = round(abs(maxyVal)/16.4);               // Update UI with max disc linear accel, convert to degree / s.

  if (abs(maxzVal) > 4000) {                        //Original 8k
    if (maxyVal < 1000) {                           //Original 2500
      settings.structure.hyzer = 1;                 //Update UI, Hyzer Release Angle
    }
    else if (maxyVal > 3500) {                      // Original 2500
      settings.structure.hyzer = 2;                // Update UI, Anhyzer Release Angle
    }
  }
  else{ 
    settings.structure.hyzer = 3;                 //Update UI, Good release angle
  }
  Serial.println("Gyro Complete");
  Serial.println((String)"G-Maxz = " + maxzVal);
  //Serial.println((String)"G-Minz = " + minzVal);
  Serial.println((String)"G-Maxy = " + maxyVal);
  Serial.println((String)"G-Maxx = " + maxxVal);
  // Y-axis max value for rotational accel
  // Negative or low y axis values for hyzer
  // Large, Positive y axis values for anhyzer

  //Strong anhyzer angle makes y positive and z negative 
 



}
